/******************************************************************************
Exploitation des mesures de la centrale inertielle de Geeros.
Ce programme est destiné à fonctionner avec Geeros Erectus,
Geeros Wifi ou Geeros Lynx, disponibles à l'adresse suivante:
http://boutique.3sigma.fr/20-geeros-erectus.html
http://boutique.3sigma.fr/21-geeros-wifi.html
http://boutique.3sigma.fr/22-geeros-lynx.html

ATTENTION: ce programme fonctionne uniquement avec les robots livrés
avant le 1er juillet 2013

Auteur: 3Sigma
Version 2.0 - 17/10/2013
*******************************************************************************/

// Librairie Arduino requise pour communiquer avec des composants sur interface I2C
#include "Wire.h"

// Inclusion d'une bibliothèque permettant de communiquer avec le capteur accélérométrique
// et gyroscopique 3 axes sur interface I2C
// Télécharger à l'adresse http://www.3sigma.fr/telechargements/I2Cdev.zip
// et décompresser dans le sous-répertoire « libraries » de votre installation Arduino
#include "I2Cdev.h"

// Inclusion d'une bibliothèque intègrant de nombreuses fonctions permettant d'accéder aux
// informations mesurées par le MPU6050, qui est le le capteur accélérométrique et
// gyroscopique 3 axes intégré à Geeros.
// Télécharger à l'adresse http://www.3sigma.fr/telechargements/MPU6050.zip
// et décompresser dans le sous-répertoire « libraries » de votre installation Arduino
#include "MPU6050_6Axis_MotionApps20.h"


// Déclaration de l'objet "capteur accélérométrique et gyroscopique"
MPU6050 mpu;

uint16_t packetSize;    // taille de paquet DMP attendu (taille par défaut: 42 octets)
uint16_t fifoCount;     // compte de tous les octets actuellement dans la FIFO
uint8_t fifoBuffer[64]; // buffer de stockage de la FIFO

Quaternion q;           // [w, x, y, z]         quaternion
VectorFloat gravity;    // [x, y, z]            vecteur de pesanteur
float euler[3];         // [psi, theta, phi]    angles d'Euler angle
float ypr[3];           // [yaw, pitch, roll]   lacet/tangage/roulis 

// Initialisations
void setup(void) {
  
  // Liaison série.
  // ATTENTION: ne pas modifier la vitesse de transmission de la liaison série,
  Serial.begin(57600);
  Serial.flush();
        
  // Initialisation du bus I2C
  Wire.begin(0x20);

  // Initialisation du capteur accélérométrique et gyroscopique
  Serial.println("Initialisation de l'IMU...");
  mpu.initialize();
  
  // Chargement et configuration du DMP
  int devStatus = mpu.dmpInitialize();
  
  if (devStatus == 0) {
    // Activation du DMP
    Serial.println(F("Activation du DMP..."));
    mpu.setDMPEnabled(true);
    
    // Acquisition de la taille de paquet DMP pour comparaison ultérieure
    packetSize = mpu.dmpGetFIFOPacketSize();
  
  } else {
    // ERREUR!
    // 1 = échec de chargement initial de la mémoire
    // 2 = échec des mises à jour de configuration du DMP
    Serial.print(F("Echec d'initialisation du DMP (code "));
    Serial.print(devStatus);
    Serial.println(F(")"));
  }
  
  // Test de la connexion
  Serial.println("Test de la connexion...");
  Serial.println(mpu.testConnection() ? "Connexion MPU6050 réussie" : "Connexion MPU6050 échouée");  

}


// Boucle principale
void loop() {

    // Acquisition du compte actuel de la FIFO
    fifoCount = mpu.getFIFOCount();
    
    // Surveillance d'overflow (qui ne devrait jamais se produire sauf si le code est inefficace)
    if (fifoCount == 1024) {
      // Reset pour pouvoir continuer proprement
      mpu.resetFIFO();
      Serial.println(F("FIFO overflow!"));
    }  
    else {
      // Attente (a priori très courte) pour des données de longueur correcte
      while (fifoCount < packetSize) {
        fifoCount = mpu.getFIFOCount();
      }
  
      // Lecture d'un paquet dans la FIFO
      mpu.getFIFOBytes(fifoBuffer, packetSize);
      
      // Mise à jour du compte de la FIFO dans le cas où il y a plus d'un paquet disponible
      // (ce qui permet d'en lire immédiatement plus sans attendre une interruption)
      fifoCount -= packetSize;
  
      // Détermination des angles d'Euler en degrés
      mpu.dmpGetQuaternion(&q, fifoBuffer);
      mpu.dmpGetGravity(&gravity, &q);
      mpu.dmpGetYawPitchRoll(ypr, &q, &gravity);

      // Envoi des données sur la liaison série suivant le protocole du programme de visualisation
      Serial.print("!");
      Serial.print("ANG:");
      Serial.print(-ypr[1] * 180/M_PI);
      Serial.print(",");
      Serial.print(ypr[2] * 180/M_PI);
      Serial.print(",");
      Serial.print(ypr[0] * 180/M_PI);
      Serial.println();    
    }

}


